package com.xiongcaiyouying.appupdate

import com.king.app.updater.AppUpdater
import com.king.app.updater.callback.UpdateCallback
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.io.File

/** AppUpdatePlugin */
class AppUpdatePlugin(private val registrar: Registrar) : EventChannel.StreamHandler {
    private var update: AppUpdater? = null

    override fun onListen(arguments: Any?, eventSink: EventChannel.EventSink?) {
        if (arguments.toString().length < 5) {
            eventSink?.error(TAG, "URL错误", arguments)
            return
        }
        if (!arguments.toString().startsWith("http")) {
            eventSink?.error(TAG, "URL错误", arguments)
            return
        }

        val data = HashMap<String, Any>()
        update = AppUpdater(registrar.context(), arguments.toString()).setUpdateCallback(object : UpdateCallback {
            override fun onFinish(file: File?) {
                data["done"] = true
                eventSink?.success(data)
            }

            override fun onDownloading(isDownloading: Boolean) {

            }

            override fun onProgress(progress: Int, total: Int, isChange: Boolean) {
                val percent = (progress * 1.0 / total * 100).toInt()
                if (isChange && percent > 0) {
                    data["percent"] = percent
                    eventSink?.success(data)
                }
            }

            override fun onCancel() {
                data["cancel"] = true
                eventSink?.success(data)
            }

            override fun onError(e: Exception?) {
                data["error"] = e.toString()
                eventSink?.success(data)
            }

            override fun onStart(url: String?) {
                data["start"] = true
                data["cancel"] = false
                data["done"] = false
                data["error"] = false
                data["percent"] = 1
                eventSink?.success(data)
            }
        })
    }

    override fun onCancel(arguments: Any?) {

    }

    companion object {
        private const val TAG = "APP_UPDATE"

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = EventChannel(registrar.messenger(), "plugins.jsonhezz.com/app_update")
            channel.setStreamHandler(AppUpdatePlugin(registrar))
        }
    }
}
